#include "ExgMgr.h"

/***********************************************************************
 *
 * FUNCTION:   WriteDBData
 *
 * DESCRIPTION: Callback for ExgDBWrite to send data with exchange manager
 *
 * PARAMETERS:  dataP : buffer containing data to send
 *                  sizeP : number of bytes to send
 *                  userDataP: app defined buffer for context
 *                  (holds exgSocket when using ExgManager)
 *
 * RETURNED:    error if non-zero
 *
 ***********************************************************************/
static Err WriteDBData(const void* dataP, ULong* sizeP, void* userDataP)
{
     Err            err;

     // Try to send as many bytes as were requested by the caller
     *sizeP = ExgSend((ExgSocketPtr)userDataP, (void*)dataP, *sizeP, &err);

     return err;
}

/***********************************************************************
 *
 * FUNCTION:        SendDatabase
 *
 * DESCRIPTION:     Sends data in the input field using the Exg API
 *
 * PARAMETERS: cardNo: card number of db to send (usually 0)
 *                  dbID:     databaseId of database to send
 *                  nameP:  public filename for this database
 *                            This is the name as it appears on a PC file listing
 *                            It should end with a .prc or .pdb extension
 *                  description: Optional text description of db to show to  user
 *                            who receives the database.
 *
 * RETURNED:        error code or zero for no error.
 ***************************************************************************/

static Err SendDatabase (Word cardNo, LocalID dbID, CharPtr nameP, CharPtr
descriptionP)
{
     ExgSocketType       exgSocket;
     Err                      err;

     // Create exgSocket structure
     MemSet(&exgSocket, sizeof(exgSocket), 0);
     exgSocket.description = descriptionP;
     exgSocket.name = nameP;

     // Start and exchange put operation
     err = ExgPut(&exgSocket);
     if (!err)
          {
          // This function converts a palm database into its external (public)
          // format. The first parameter is a callback that will be passed parts of
          // the database to send or write.
          err = ExgDBWrite(WriteDBData, &exgSocket, NULL, dbID, cardNo);
          // Disconnect Exg and pass error
          err = ExgDisconnect(&exgSocket, err);
          }
     return err;
}

/***********************************************************************
 *
 * FUNCTION:        SendMe
 *
 * DESCRIPTION:     Sends this application
 *
 * PARAMETERS: none
 *
 * RETURNED:   error code or zero for no error.
 *
 ***************************************************************************/
static Err SendMe(char* file_name, char* send_name, char* send_description)
{
     Err err;
     // Find our app using its internal name
     LocalID dbID = DmFindDatabase(0, file_name);
     if (dbID)    // send it giving external name and description
          err = SendDatabase(0, dbID, send_name, send_description);
     else
          err = DmGetLastErr();
     return err;
}

//  file_name is the name of the file to be sent
//  send_name is the name it is being sent as and must end in .prc or .pdb
//  send_description is the text that appears while beaming
double beam_filE_cf_(char* file_name, char* send_name, char* send_description)
{
     Err err;
     err = SendMe(file_name, send_name, send_description);

     // Must free strings that are passed into the function
     _pda_any_dispose(file_name);
     _pda_any_dispose(send_name);
     _pda_any_dispose(send_description);

     // Convert the error to a float and return it
     return _pda_sdword_to_float(err);
}

